"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkService = void 0;
const common_1 = require("@nestjs/common");
const ethers_1 = require("ethers");
const network_constant_1 = require("./network.constant");
const network_entity_1 = require("./network.entity");
const typeorm_1 = require("@nestjs/typeorm");
const providers_1 = require("@ethersproject/providers");
const crud_typeorm_1 = require("@nestjsx/crud-typeorm");
const typeorm_2 = require("typeorm");
let NetworkService = class NetworkService extends crud_typeorm_1.TypeOrmCrudService {
    constructor(repo) {
        super(repo);
        this.repo = repo;
        this._networkWithProviderByChainKey = new Map();
        this.entity = network_entity_1.Network;
        this.relations = network_entity_1.Network.relations;
        this.recursiveRelations = network_entity_1.Network.recursiveRelations;
    }
    async onModuleInit() {
        await this.refreshNetwork();
    }
    async findOneBy(where, manager) {
        const options = {
            where,
            relations: [...this.relations, ...this.recursiveRelations],
        };
        if (manager) {
            return manager.findOne(this.entity, options);
        }
        return this.findOne(options);
    }
    async findAllBy(where, manager) {
        const options = {
            where,
            relations: [...this.relations, ...this.recursiveRelations],
        };
        if (manager) {
            return manager.find(this.entity, options);
        }
        return this.find(options);
    }
    async createOneBy(params, manager) {
        const createEntity = this.repo.create(params);
        if (manager) {
            return manager.save(this.entity, createEntity);
        }
        return this.repo.save(createEntity);
    }
    async createAllBy(params, manager) {
        const createEntities = params.map((param) => this.repo.create(param));
        if (manager) {
            return manager.save(this.entity, createEntities);
        }
        return this.repo.save(createEntities);
    }
    async createAllIfNotExistBy(params, manager) {
        let queryBuilder;
        if (manager) {
            queryBuilder = manager.createQueryBuilder();
        }
        else {
            queryBuilder = this.repo.createQueryBuilder();
        }
        const createEntities = params.map((param) => this.repo.create(param));
        return queryBuilder
            .insert()
            .into(network_entity_1.Network)
            .values(createEntities)
            .orIgnore()
            .execute();
    }
    async updateOneBy(where, set, manager) {
        if (manager) {
            return manager.update(this.entity, where, set);
        }
        return this.repo.update(where, set);
    }
    async deleteOneBy(where, manager) {
        if (manager) {
            return manager.delete(this.entity, where);
        }
        return this.repo.delete(where);
    }
    provider(chainKey) {
        const providers = this.providers(chainKey);
        return new providers_1.FallbackProvider([...providers]);
        return providers[Math.floor(Math.random() * providers.length)];
    }
    providers(chainKey) {
        return this._networkWithProviderByChainKey.get(chainKey).providers;
    }
    async refreshNetwork(network) {
        const refreshNetworks = network ? [network] : await this.findAllBy({});
        refreshNetworks.forEach((network) => {
            const providers = this.generateNetworkProviders(network);
            this._networkWithProviderByChainKey.set(network.chainKey, Object.assign(Object.assign({}, network), { providers, chainKey: network.chainKey }));
        });
    }
    multiCallAddress(chainKey) {
        return this._networkWithProviderByChainKey.get(chainKey).multiCallAddress;
    }
    generateNetworkProviders(network) {
        const providers = [];
        const { chainType, chainId, http, name } = network;
        switch (chainType) {
            case network_constant_1.NETWORK_CHAIN_TYPE.EVM: {
                for (const { url, provide, user, headers } of http) {
                    if (provide) {
                        const provider = new ethers_1.ethers.providers.StaticJsonRpcProvider({
                            url,
                            headers,
                            user,
                        }, {
                            name,
                            chainId: parseInt(chainId),
                        });
                        providers.push(provider);
                    }
                }
                break;
            }
            case network_constant_1.NETWORK_CHAIN_TYPE.SOLANA:
                for (const { url, provide, user, headers } of http) {
                    if (provide) {
                        const provider = new ethers_1.ethers.providers.StaticJsonRpcProvider({
                            url,
                            headers,
                            user,
                        }, {
                            name,
                            chainId: parseInt(chainId),
                        });
                        providers.push(provider);
                    }
                }
                break;
        }
        return providers;
    }
    async checkProviderHealth(network) {
        const checkNetworks = network ? [network] : await this.findAllBy({});
        const totalState = {};
        await Promise.all(checkNetworks.map(async (network) => {
            var e_1, _a;
            const { subName, chainKey, chainType } = network;
            const { providers } = this._networkWithProviderByChainKey.get(chainKey);
            totalState[subName] = [];
            switch (chainType) {
                case network_constant_1.NETWORK_CHAIN_TYPE.EVM: {
                    try {
                        for (var providers_2 = __asyncValues(providers), providers_2_1; providers_2_1 = await providers_2.next(), !providers_2_1.done;) {
                            const provider = providers_2_1.value;
                            const { url } = JSON.parse(JSON.stringify(provider)).connection;
                            let health = false;
                            try {
                                await provider.getBlockNumber();
                                health = true;
                            }
                            catch (e) {
                                health = false;
                            }
                            totalState[subName].push({ url, health });
                        }
                    }
                    catch (e_1_1) { e_1 = { error: e_1_1 }; }
                    finally {
                        try {
                            if (providers_2_1 && !providers_2_1.done && (_a = providers_2.return)) await _a.call(providers_2);
                        }
                        finally { if (e_1) throw e_1.error; }
                    }
                    break;
                }
            }
        }));
        return totalState;
    }
};
NetworkService = __decorate([
    (0, common_1.Injectable)(),
    __param(0, (0, typeorm_1.InjectRepository)(network_entity_1.Network)),
    __metadata("design:paramtypes", [typeorm_2.Repository])
], NetworkService);
exports.NetworkService = NetworkService;
//# sourceMappingURL=network.service.js.map